%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2019-06-18
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/ustereo.html
%
% (c) 2019 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2019-06-08  1.0.20190608  Original Version
%

function ln_rho = circularityNorthPoleConstraint(p, q, lambda)
[rows, cols] = size(p);

pointCount = rows*cols;
eqCount = 2 * ((rows-1) * cols) + rows;

M = sparse(eqCount, pointCount);
b = zeros(eqCount, 1);

eqId = 0;
inverseOfdTheta = lambda/pi;

for row = 1:rows-1
    for col = 1:cols-1
        eqId = eqId + 1;

        M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
        M(eqId, (row-1)*cols + col + 1) = inverseOfdTheta;
        b(eqId, 1) = q(row, col);
        
        eqId = eqId + 1;
        M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
        M(eqId, row*cols + col) = inverseOfdTheta;
        b(eqId, 1) = p(row, col);
    end
    
    col = cols;
    eqId = eqId + 1;
    M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
    M(eqId, (row-1)*cols + 1) = inverseOfdTheta;
    b(eqId, 1) = q(row, col);
   
    eqId = eqId + 1;
    M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
    M(eqId, row*cols + col) = inverseOfdTheta;
    b(eqId, 1) = p(row, col);
end

row = rows;
for col = 1:cols-1
    eqId = eqId + 1;
    M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
    M(eqId, (row-1)*cols + col + 1) = inverseOfdTheta;
    b(eqId, 1) = q(row, col);
end

col = cols;
eqId = eqId + 1;
M(eqId, (row-1)*cols + col) = -inverseOfdTheta;
M(eqId, (row-1)*cols + 1) = inverseOfdTheta;
b(eqId, 1) = q(row, col);

X = M \ b;

ln_rho = zeros(row, col);
eqId = 0;

for row = 1:rows
    for col = 1:cols
        eqId = eqId + 1;
        ln_rho(row, col) = X(eqId, 1);
    end
end

