%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2019-06-18
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/ustereo.html
%
% (c) 2019 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2019-06-08  1.0.20190608  Original Version
%

function [ p, q ] = computePQ(I, L, theta, phi)
    n = length(I);

    I = double(I);
    
    A = [];
    B = [];
    C = [];
    
    if theta == 0
        p = 0; q = 0;
        return
    end
    
    R = rotoz(phi) * rotoy(theta);
    e_theta = R(:, 1);
    e_phi = R(:, 2);
    e_rho = R(:, 3);
    
    for i = 1 : n-1
        for k = i+1 : n
            Ii = I(i);
            Ik = I(k);
            
            if Ii ~= 0 && Ik ~= 0         
            %if Ii > 0.1 && Ik > 0.1   
                Li = L(:, i);
                Lk = L(:, k);

                A = [ A ; Ii * norm(Li) * dot(Lk, e_theta) - Ik * norm(Lk) * dot(Li, e_theta) ];
                B = [ B ; (Ii * norm(Li) * dot(Lk, e_phi) - Ik * norm(Lk) * dot(Li, e_phi)) / sin(theta) ];
                C = [ C ; -Ii * norm(Li) * dot(Lk, e_rho) + Ik * norm(Lk) * dot(Li, e_rho) ];
            end
        end
    end
    
    s = length(A);

    if s >= 3
        M = pinv([ A B ]) * C;

        p = M(1);
        q = M(2);
    else
        %warning('Unable to compute p and q')
        p = 0; q = 0;
    end
end
