%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2019-06-18
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/ustereo.html
%
% (c) 2019 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2019-06-08  1.0.20190608  Original Version
%

function [engine, rho, p, q] = computeRho(engine, varargin)
fprintf('Estimating rho... ');

phi = engine.parameters.phi;
theta = engine.parameters.theta;

[m, n] = size(engine.estimation);

p = zeros(m, n);
q = zeros(m, n);

for i = 1 : m
    for j = 1 : n
        p(i, j) = engine.estimation{i, j}.p;
        q(i, j) = engine.estimation{i, j}.q;
    end
end

% Methods:
% - Circularity constraint
% - Circularity constraint + North-pole constraint

if nargin == 1
    method = 0;
else
    method = varargin{1};
end

if nargin == 6
    min_u = round(varargin{2});
    max_u = round(varargin{3});
    min_v = round(varargin{4});
    max_v = round(varargin{5});
else
    min_u = 1;
    max_u = n;
    min_v = 1;
    max_v = m;
end

p = p(min_v:max_v, min_u:max_u);
q = q(min_v:max_v, min_u:max_u);
phi = phi(min_u:max_u);
theta = theta(min_v:max_v);

% NORMAL
if method == 0
    fprintf('[CircularityConstraint] ');
    ln_rho = circularityConstraint(p, q, engine.parameters.lambda);
elseif method == 1
    fprintf('[CircularityConstraint+NorthPoleConstraint] ');
    ln_rho = circularityNorthPoleConstraint(p, q, engine.parameters.lambda);
else
    fprintf('\n');
    error('Unknown method selected');
end

rho = exp(ln_rho);
rhoNorm = (rho - min(rho(:))) / (max(rho(:)) - min(rho(:)));
%rhoNorm = (rho - mean(rho(:))) / (max(rho(:)) - min(rho(:)));

for i = 1 : m
    for j = 1 : n
        engine.estimation{i, j}.rho = rho(i, j);
        engine.estimation{i, j}.rhoNorm = rhoNorm(i, j);
    end
end

fprintf('OK!\n')
