%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2019-06-18
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/ustereo.html
%
% (c) 2019 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2019-06-08  1.0.20190608  Original Version
%

function [engine] = generateAnalyticData(rhoExpression, lambda, lights, noiseStdDev)
noise = {};
noise.mean = 0;
noise.standard_deviation = noiseStdDev;
noise.variance = noiseStdDev^2;

theta = pi/lambda : pi/lambda : pi/2 - pi/lambda;
phi = 0 : pi/lambda : 2*pi - pi/lambda;

syms t p pd qd

engine = {};
engine.parameters = {};
engine.parameters.lambda = lambda;
engine.parameters.noise = noise;
engine.parameters.theta = theta;
engine.parameters.phi = phi;
engine.parameters.lights = lights;
engine.generation = {};
engine.generation.rho = rhoExpression(t, p);

engine.generation.X = ...
[
    rhoExpression(t, p) .* sin(t) .* cos(p) ;
    rhoExpression(t, p) .* sin(t) .* sin(p) ;
    rhoExpression(t, p) .* cos(t)
];

engine.generation.N = ...
    cross( ...
        diff(engine.generation.X, 'p'), ...
        diff(engine.generation.X, 't') ...
    );
engine.generation.N = engine.generation.N / norm(engine.generation.N);

L = [ pd ; qd ; -1 ];
L = L / norm(L);

engine.generation.I = dot(engine.generation.N, L);

engine.data = {};

[p, t] = meshgrid(phi, theta);

Nx = eval(engine.generation.N(1));
if length(Nx(:)) < length(theta)*length(phi)
    Nx = zeros(length(theta), length(phi));
end
Ny = eval(engine.generation.N(2));
if length(Ny(:)) < length(theta)*length(phi)
    Ny = zeros(length(theta), length(phi));
end
Nz = eval(engine.generation.N(3));
if length(Nz(:)) < length(theta)*length(phi)
    Nz = zeros(length(theta), length(phi));
end

rawI = {};
I = {};
rho = eval(rhoExpression);
rhoNorm = (rho - min(rho(:))) / (max(rho(:)) - min(rho(:)));
[~,lights_count] = size(lights);

for l = 1:lights_count
    pd = lights(1, l);
    qd = lights(2, l);

    rawI{l} = eval(engine.generation.I);%double(floor(eval(engine.generation.I) * 255)) / 255;
    
    I{l} = rawI{l} + rand(size(rawI{l})) * noise.variance;

    I{l}(I{l} > 1) = 1;
    I{l}(I{l} < 0) = 0;
end

for i = 1:length(theta)
    for j = 1:length(phi)
        t = theta(i);
        p = phi(j);
        
        engine.data{i, j}.N = [ Nx(i, j) ; Ny(i, j) ; Nz(i, j) ];
        engine.data{i, j}.rho = rho(i, j);
        engine.data{i, j}.rhoNorm = rhoNorm(i, j);
        
        engine.data{i, j}.rawI = zeros(lights_count, 1);
        engine.data{i, j}.I = zeros(lights_count, 1);
        
        for l = 1:lights_count
            engine.data{i, j}.rawI(l) = rawI{l}(i, j);
            engine.data{i, j}.I(l) = I{l}(i, j);
        end
    end
end

