%
% References :
%   -
%
% Author:  Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib
% Date:    2019-06-18
% Version: 1.0
% Umail:   jordan.caracotte@etud.u-picardie.fr
% URL:     https://mis.u-picardie.fr/~fabio/ustereo.html
%
% (c) 2019 Jordan Caracotte, Fabio Morbidi and El Mustapha Mouaddib, 
% MIS Laboratory, University of Picardie Jules Verne, Amiens, France
%
% History:
%   Date:       Version       Comment:
%   2019-06-08  1.0.20190608  Original Version
%

clear; close all

syms t p

%rho(t, p) = 9/cos(t) + 100*sin(t); % Volcano
rho(t, p) = 6 + sin(t/10 + 5*p);  % Starfish

lights = ...
[
    0.25,  0   , -0.25,  0   , -0.25, -0.25,  0.25,  0.25 ;
    0   ,  0.25,  0   , -0.25, -0.25,  0.25, -0.25,  0.25 ;
   -1   , -1   , -1   , -1   , -1   , -1   , -1   , -1    ;
];

lambda = 100;

engine = generateAnalyticData(rho, lambda, lights, 0.3);
%[ filename, pathname ] = uigetfile({ '*.gndtrth' }, 'Select groundtruth file');
%engine = generateNumericData([ pathname, filename ], lights, 0.3, 1);

engine = computeNormals(engine);
[engine, rho, p, q] = computeRho(engine, 1);
engine = computeErrors(engine);

phi = engine.parameters.phi;
theta = engine.parameters.theta;
[phi, theta] = meshgrid(phi, theta);

X = rho.*sin(theta).*cos(phi);
Y = rho.*sin(theta).*sin(phi);
Z = rho.*cos(theta);

figure
s = surfl(X, Y, Z);
shading faceted
colormap gray
s.EdgeColor = 'none';
axis equal

figure
pcshow([X(:), Y(:), Z(:)])

